{-# OPTIONS_GHC -fglasgow-exts #-}

module Nat where {

import qualified HOL;
import qualified Product_Type;

data Nat = Zero_nat | Suc Nat.Nat;

plus_nat :: Nat.Nat -> Nat.Nat -> Nat.Nat;
plus_nat (Nat.Suc m) n = Nat.plus_nat m (Nat.Suc n);
plus_nat Nat.Zero_nat n = n;

one_nat :: Nat.Nat;
one_nat = Nat.Suc Nat.Zero_nat;

less_eq_nat :: Nat.Nat -> Nat.Nat -> Bool;
less_eq_nat (Nat.Suc m) n = Nat.less_nat m n;
less_eq_nat Nat.Zero_nat n = True;

less_nat :: Nat.Nat -> Nat.Nat -> Bool;
less_nat m (Nat.Suc n) = Nat.less_eq_nat m n;
less_nat n Nat.Zero_nat = False;

minus_nat :: Nat.Nat -> Nat.Nat -> Nat.Nat;
minus_nat (Nat.Suc m) (Nat.Suc n) = Nat.minus_nat m n;
minus_nat Nat.Zero_nat n = Nat.Zero_nat;
minus_nat m Nat.Zero_nat = m;

nat_case :: forall t. t -> (Nat.Nat -> t) -> Nat.Nat -> t;
nat_case f1 f2 (Nat.Suc nat) = f2 nat;
nat_case f1 f2 Nat.Zero_nat = f1;

eq_nat :: Nat.Nat -> Nat.Nat -> Bool;
eq_nat (Nat.Suc a) Nat.Zero_nat = False;
eq_nat Nat.Zero_nat (Nat.Suc a) = False;
eq_nat (Nat.Suc nat) (Nat.Suc nat') = Nat.eq_nat nat nat';
eq_nat Nat.Zero_nat Nat.Zero_nat = True;

instance Eq Nat.Nat where {
  a == b = Nat.eq_nat a b;
};

times_nat :: Nat.Nat -> Nat.Nat -> Nat.Nat;
times_nat Nat.Zero_nat n = Nat.Zero_nat;
times_nat (Nat.Suc m) n = Nat.plus_nat n (Nat.times_nat m n);

divmod :: Nat.Nat -> Nat.Nat -> (Nat.Nat, Nat.Nat);
divmod m n =
  (if n == Nat.Zero_nat || Nat.less_nat m n then (Nat.Zero_nat, m)
    else let {
           a = Nat.divmod (Nat.minus_nat m n) n;
           (q, aa) = a;
         } in (Nat.Suc q, aa));

mod_nat :: Nat.Nat -> Nat.Nat -> Nat.Nat;
mod_nat m n = snd (Nat.divmod m n);

div_nat :: Nat.Nat -> Nat.Nat -> Nat.Nat;
div_nat m n = fst (Nat.divmod m n);

}
