{-# OPTIONS_GHC -fglasgow-exts #-}

module ProofChecker where {

import qualified Show;
import qualified Term;
import qualified Loop;
import qualified DpTransImpl;
import qualified List;
import qualified ErrorMonad;
import qualified Polynomial;
import qualified DepGraph;
import qualified Option;
import qualified Nat;
import qualified Util;
import qualified Trs;

data TRSDisproof a b =
  Loop ([(Term.Term a b)], (Term.Ctxt a b, [(Term.Term a b, Term.Term a b)]))
  | NotWellFormed;

data DPProof a b = PisEmpty
  | RedPairProc (Polynomial.RedPair a) [(Term.Term a b, Term.Term a b)]
      (ProofChecker.DPProof a b)
  | RedPairUrProc (Polynomial.RedPair a) [(Term.Term a b, Term.Term a b)]
      [(Term.Term a b, Term.Term a b)] (ProofChecker.DPProof a b)
  | DepGraphProc
      [(Maybe (ProofChecker.DPProof a b), [(Term.Term a b, Term.Term a b)])]
  | MonoRedPairProc (Polynomial.RedPair a) [(Term.Term a b, Term.Term a b)]
      [(Term.Term a b, Term.Term a b)] (ProofChecker.DPProof a b)
  | MonoRedPairUrProc (Polynomial.RedPair a) [(Term.Term a b, Term.Term a b)]
      [(Term.Term a b, Term.Term a b)] [(Term.Term a b, Term.Term a b)]
      (ProofChecker.DPProof a b);

data TRSProof a b =
  DPTrans [(Term.Term (Trs.Shp a) b, Term.Term (Trs.Shp a) b)]
    (ProofChecker.DPProof (Trs.Shp a) b)
  | RuleRemoval (Polynomial.RedPair a) [(Term.Term a b, Term.Term a b)]
      (ProofChecker.TRSProof a b)
  | RisEmpty;

indent :: [[Char]] -> [[Char]];
indent xs = Show.newline : List.mapa (List.append [' ', ' ']) xs;

checkLoop ::
  forall a b.
    (Eq a, Show.Sshow a, Eq b,
      Show.Sshow b) => [(Term.Term a b, Term.Term a b)] ->
                         ([(Term.Term a b)],
                           (Term.Ctxt a b, [(Term.Term a b, Term.Term a b)]))
                           -> Util.Eithera [[Char]] ();
checkLoop r (ts, (c, sub)) =
  ErrorMonad.catcha
    (case Loop.subst_safe [] sub (Term.Sub Term.Var) of {
      Nothing ->
        ErrorMonad.errora
          ['s', 'u', 'b', 's', 't', 'i', 't', 'u', 't', 'i', 'o', 'n', ' ', 'n',
            'o', 't', ' ', 'u', 'n', 'i', 'q', 'u', 'e'];
      Just sigma -> Loop.check_loop (ts, (c, sigma)) r;
    })
    (\ x ->
      ErrorMonad.errora
        [['e', 'r', 'r', 'o', 'r', ' ', 'i', 'n', ' ', 'c', 'e', 'r', 't', 'i',
           'f', 'i', 'c', 'a', 't', 'i', 'o', 'n', ' ', 'o', 'f', ' ', 'l', 'o',
           'o', 'p'],
          x]);

checkTRSDisproof ::
  forall a b.
    (Eq a, Show.Sshow a, Eq b,
      Show.Sshow b) => [Char] ->
                         [(Term.Term a b, Term.Term a b)] ->
                           ProofChecker.TRSDisproof a b ->
                             Util.Eithera [[Char]] ();
checkTRSDisproof i r (ProofChecker.Loop l) =
  ErrorMonad.catcha (ProofChecker.checkLoop r l)
    (\ x ->
      ErrorMonad.errora
        (List.append i
           [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'i', 'n', ' ', 'l', 'o',
             'o', 'p', ' ', 'p', 'r', 'o', 'o', 'f']
          : x));
checkTRSDisproof i r ProofChecker.NotWellFormed =
  ErrorMonad.catcha (DpTransImpl.checkNotWellFormed r)
    (\ x ->
      ErrorMonad.errora
        (List.append i
           [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'i', 'n', ' ', 'n', 'o',
             't', '-', 'w', 'e', 'l', 'l', '-', 'f', 'o', 'r', 'm', 'e', 'd',
             ' ', 'p', 'r', 'o', 'o', 'f']
          : x));

checkDPProof ::
  forall a b.
    (Eq a, Show.Sshow a, Eq b,
      Show.Sshow b) => [Char] ->
                         [(Term.Term a b, Term.Term a b)] ->
                           [(Term.Term a b, Term.Term a b)] ->
                             ProofChecker.DPProof a b ->
                               Util.Eithera [[Char]] ();
checkDPProof i p r ProofChecker.PisEmpty =
  (case p of {
    [] -> ErrorMonad.returna ();
    x : xa ->
      ErrorMonad.errora
        (List.append i
           [':', ' ', 'P', ' ', 'i', 's', ' ', 'n', 'o', 't', ' ', 'e', 'm',
             'p', 't', 'y', ' ', 'i', 'n', ' ', 't', 'h', 'e', ' ', 'f', 'o',
             'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'D', 'P', '-', 'p', 'r',
             'o', 'b', 'l', 'e', 'm', ' ']
          : Trs.showDPP p r);
  });
checkDPProof i p r (ProofChecker.RedPairProc redp rP prf) =
  ErrorMonad.bind
    (ErrorMonad.catcha (Polynomial.createRedPairProc redp p rP r)
      (\ x ->
        ErrorMonad.errora
          (List.append i
             [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n', ' ',
               'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ', 't', 'h', 'e', ' ',
               'r', 'e', 'd', 'u', 'c', 't', 'i', 'o', 'n', ' ', 'p', 'a', 'i',
               'r', ' ', 'p', 'r', 'o', 'c', 'e', 's', 's', 'o', 'r', ' ', 't',
               'o', ' ', 's', 'w', 'i', 't', 'c', 'h', ' ', 'f', 'r', 'o', 'm',
               ' ']
            : List.append (Trs.showDPP p r)
                (['t', 'o', ' ', 't', 'h', 'e', ' ', 'D', 'P', ' ', 'p', 'r',
                   'o', 'b', 'l', 'e', 'm', ' ']
                  : List.append (Trs.showDPP rP r) x))))
    (\ x ->
      ErrorMonad.catcha
        (ProofChecker.checkDPProof (List.append i ['.', '1']) rP r prf)
        (\ xa ->
          ErrorMonad.errora
            (List.append i
               [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o', 'w',
                 ' ', 't', 'h', 'e', ' ', 'r', 'e', 'd', 'u', 'c', 't', 'i',
                 'o', 'n', ' ', 'p', 'a', 'i', 'r', ' ', 'p', 'r', 'o', 'c',
                 'e', 's', 's', 'o', 'r']
              : ProofChecker.indent xa)));
checkDPProof i p r (ProofChecker.RedPairUrProc redp rP ur prf) =
  ErrorMonad.bind
    (ErrorMonad.catcha (Polynomial.createRedPairUrAfProc redp p rP r ur)
      (\ x ->
        ErrorMonad.errora
          (List.append i
             [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n', ' ',
               'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ', 't', 'h', 'e', ' ',
               'r', 'e', 'd', 'u', 'c', 't', 'i', 'o', 'n', ' ', 'p', 'a', 'i',
               'r', ' ', 'p', 'r', 'o', 'c', 'e', 's', 's', 'o', 'r', ' ', '(',
               'w', 'i', 't', 'h', ' ', 'u', 's', 'a', 'b', 'l', 'e', ' ', 'r',
               'u', 'l', 'e', 's', ' ', 'a', 'n', 'd', ' ', 'a', 'f', ')', ' ',
               't', 'o', ' ', 's', 'w', 'i', 't', 'c', 'h', ' ', 'f', 'r', 'o',
               'm', ' ']
            : List.append (Trs.showDPP p r)
                (['t', 'o', ' ', 't', 'h', 'e', ' ', 'D', 'P', ' ', 'p', 'r',
                   'o', 'b', 'l', 'e', 'm', ' ']
                  : List.append (Trs.showDPP rP r) x))))
    (\ x ->
      ErrorMonad.catcha
        (ProofChecker.checkDPProof (List.append i ['.', '1']) rP r prf)
        (\ xa ->
          ErrorMonad.errora
            (List.append i
               [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o', 'w',
                 ' ', 't', 'h', 'e', ' ', 'r', 'e', 'd', 'u', 'c', 't', 'i',
                 'o', 'n', ' ', 'p', 'a', 'i', 'r', ' ', 'p', 'r', 'o', 'c',
                 'e', 's', 's', 'o', 'r']
              : ProofChecker.indent xa)));
checkDPProof i p r (ProofChecker.DepGraphProc edpts) =
  ErrorMonad.bind
    (ErrorMonad.catcha
      (DepGraph.checkDepGraphProc p
        (List.mapa
          (\ x ->
            ((case fst x of {
               Nothing -> False;
               Just xa -> True;
             }),
              snd x))
          edpts)
        r)
      (\ x ->
        ErrorMonad.errora
          (List.append i
             [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'i', 'l', 'e',
               ' ', 't', 'r', 'y', 'i', 'n', 'g', ' ', 't', 'o', ' ', 'p', 'e',
               'r', 'f', 'o', 'r', 'm', ' ', 'S', 'C', 'C', '-', 'd', 'e', 'c',
               'o', 'm', 'p', 'o', 's', 'i', 't', 'i', 'o', 'n', ' ', ' ', 'o',
               'n', ' ']
            : List.append (Trs.showDPP p r) x)))
    (\ x ->
      ErrorMonad.catcha
        (ErrorMonad.checkAllMIndex
          (\ j rp ->
            (case fst rp of {
              Nothing -> j;
              Just prf -> Nat.Suc j;
            }))
          (\ j rp ->
            (case fst rp of {
              Nothing -> ErrorMonad.returna ();
              Just a ->
                ProofChecker.checkDPProof
                  (List.append (List.append i ['.']) (Show.sshow_nat j))
                  (snd rp) r a;
            }))
          edpts)
        (\ xa ->
          ErrorMonad.errora
            (List.append i
               [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o', 'w',
                 ' ', 't', 'h', 'e', ' ', 'd', 'e', 'p', 'e', 'n', 'd', 'e',
                 'n', 'c', 'y', ' ', 'g', 'r', 'a', 'p', 'h', ' ', 'p', 'r',
                 'o', 'c', 'e', 's', 's', 'o', 'r']
              : ProofChecker.indent xa)));
checkDPProof i p r (ProofChecker.MonoRedPairProc redp rP rR prf) =
  ErrorMonad.bind
    (ErrorMonad.catcha (Polynomial.createMonoRedPairProc redp p rP r rR)
      (\ x ->
        ErrorMonad.errora
          (List.append i
             [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n', ' ',
               'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ', 't', 'h', 'e', ' ',
               'm', 'o', 'n', 'o', 't', 'o', 'n', 'i', 'c', ' ', 'r', 'e', 'd',
               'u', 'c', 't', 'i', 'o', 'n', ' ', 'p', 'a', 'i', 'r', ' ', 'p',
               'r', 'o', 'c', 'e', 's', 's', 'o', 'r', ' ', 't', 'o', ' ', 's',
               'w', 'i', 't', 'c', 'h', ' ', 'f', 'r', 'o', 'm', ' ']
            : List.append (Trs.showDPP p r)
                (['t', 'o', ' ', 't', 'h', 'e', ' ', 'D', 'P', ' ', 'p', 'r',
                   'o', 'b', 'l', 'e', 'm', ' ']
                  : List.append (Trs.showDPP rP rR) x))))
    (\ x ->
      ErrorMonad.catcha
        (ProofChecker.checkDPProof (List.append i ['.', '1']) rP rR prf)
        (\ xa ->
          ErrorMonad.errora
            (List.append i
               [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o', 'w',
                 ' ', 't', 'h', 'e', ' ', 'm', 'o', 'n', 'o', 't', 'o', 'n',
                 'i', 'c', ' ', 'r', 'e', 'd', 'u', 'c', 't', 'i', 'o', 'n',
                 ' ', 'p', 'a', 'i', 'r', ' ', 'p', 'r', 'o', 'c', 'e', 's',
                 's', 'o', 'r']
              : ProofChecker.indent xa)));
checkDPProof i p r (ProofChecker.MonoRedPairUrProc redp rP rR ur prf) =
  ErrorMonad.bind
    (ErrorMonad.catcha (Polynomial.createMonoRedPairUrProc redp p rP r rR ur)
      (\ x ->
        ErrorMonad.errora
          (List.append i
             [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n', ' ',
               'a', 'p', 'p', 'l', 'y', 'i', 'n', 'g', ' ', 't', 'h', 'e', ' ',
               'm', 'o', 'n', 'o', 't', 'o', 'n', 'i', 'c', ' ', 'r', 'e', 'd',
               'u', 'c', 't', 'i', 'o', 'n', ' ', 'p', 'a', 'i', 'r', ' ', 'p',
               'r', 'o', 'c', 'e', 's', 's', 'o', 'r', ' ', '(', 'w', 'i', 't',
               'h', ' ', 'u', 's', 'a', 'b', 'l', 'e', ' ', 'r', 'u', 'l', 'e',
               's', ')', ' ', 't', 'o', ' ', 's', 'w', 'i', 't', 'c', 'h', ' ',
               'f', 'r', 'o', 'm', ' ']
            : List.append (Trs.showDPP p r)
                (['t', 'o', ' ', 't', 'h', 'e', ' ', 'D', 'P', ' ', 'p', 'r',
                   'o', 'b', 'l', 'e', 'm', ' ']
                  : List.append (Trs.showDPP rP rR) x))))
    (\ x ->
      ErrorMonad.catcha
        (ProofChecker.checkDPProof (List.append i ['.', '1']) rP rR prf)
        (\ xa ->
          ErrorMonad.errora
            (List.append i
               [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o', 'w',
                 ' ', 't', 'h', 'e', ' ', 'm', 'o', 'n', 'o', 't', 'o', 'n',
                 'i', 'c', ' ', 'r', 'e', 'd', 'u', 'c', 't', 'i', 'o', 'n',
                 ' ', 'p', 'a', 'i', 'r', ' ', '(', 'w', 'i', 't', 'h', ' ',
                 'u', 's', 'a', 'b', 'l', 'e', ' ', 'r', 'u', 'l', 'e', 's',
                 ')', ' ', 'p', 'r', 'o', 'c', 'e', 's', 's', 'o', 'r']
              : ProofChecker.indent xa)));

checkTRSProof ::
  forall a b.
    (Eq a, Show.Sshow a, Eq b,
      Show.Sshow b) => [Char] ->
                         [(Term.Term a b, Term.Term a b)] ->
                           ProofChecker.TRSProof a b ->
                             Util.Eithera [[Char]] ();
checkTRSProof i r (ProofChecker.DPTrans p prf) =
  ErrorMonad.bind
    (ErrorMonad.catcha (DpTransImpl.checkDPs r p)
      (\ x ->
        ErrorMonad.errora
          (List.append i
             [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'w', 'h', 'e', 'n', ' ',
               's', 'w', 'i', 't', 'c', 'h', 'i', 'n', 'g', ' ', 'f', 'r', 'o',
               'm', ' ', 't', 'h', 'e', ' ', 'T', 'R', 'S']
            : List.append (Trs.showTRS r)
                (['t', 'o', ' ', 't', 'h', 'e', ' ', 'i', 'n', 'i', 't', 'i',
                   'a', 'l', ' ', 'D', 'P', ' ', 'p', 'r', 'o', 'b', 'l', 'e',
                   'm']
                  : List.append (Trs.showDPP p (DpTransImpl.computeID r)) x))))
    (\ x ->
      ErrorMonad.catcha
        (ProofChecker.checkDPProof (List.append i ['.', '1']) p
          (DpTransImpl.computeID r) prf)
        (\ xa ->
          ErrorMonad.errora
            (List.append i
               [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o', 'w',
                 ' ', 's', 'w', 'i', 't', 'c', 'h', ' ', 't', 'o', ' ', 'd',
                 'e', 'p', 'e', 'n', 'd', 'e', 'n', 'c', 'y', ' ', 'p', 'a',
                 'i', 'r', 's']
              : ProofChecker.indent xa)));
checkTRSProof i r (ProofChecker.RuleRemoval rp rR prfa) =
  ErrorMonad.bind
    (ErrorMonad.catcha (Polynomial.createRuleRemovalProc rp r rR)
      (\ x ->
        ErrorMonad.errora
          (List.append i
             [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'i', 'n', ' ', 'r', 'u',
               'l', 'e', ' ', 'r', 'e', 'm', 'o', 'v', 'a', 'l', ' ', 'w', 'h',
               'e', 'n', ' ', 's', 'w', 'i', 't', 'c', 'h', 'i', 'n', 'g', ' ',
               'f', 'r', 'o', 'm', ' ', 't', 'h', 'e', ' ', 'T', 'R', 'S']
            : List.append (Trs.showTRS r)
                (['t', 'o', ' ', 't', 'h', 'e', ' ', 'T', 'R', 'S'] :
                  List.append (Trs.showTRS rR) x))))
    (\ x ->
      ErrorMonad.catcha
        (ProofChecker.checkTRSProof (List.append i ['.', '1']) rR prfa)
        (\ xa ->
          ErrorMonad.errora
            (List.append i
               [':', ' ', 'e', 'r', 'r', 'o', 'r', ' ', 'b', 'e', 'l', 'o', 'w',
                 ' ', 't', 'h', 'e', ' ', 'r', 'u', 'l', 'e', ' ', 'r', 'e',
                 'm', 'o', 'v', 'a', 'l', ' ', 'p', 'r', 'o', 'c', 'e', 's',
                 's', 'o', 'r']
              : ProofChecker.indent xa)));
checkTRSProof i r ProofChecker.RisEmpty =
  ErrorMonad.checkB (List.nulla r)
    (List.append
      [List.append i
         [':', ' ', 'R', ' ', 'i', 's', ' ', 'n', 'o', 't', ' ', 'e', 'm', 'p',
           't', 'y', ',', ' ', 'b', 'u', 't', ' ', 'i', 't', ' ', 'i', 's', ' ',
           't', 'h', 'e', ' ', 'T', 'R', 'S']]
      (Trs.showTRS r));

}
