{-# OPTIONS_GHC -fglasgow-exts #-}

module Xml where {

import qualified Show;
import qualified Nat;
import qualified List;
import qualified Util;
import qualified Term;
import qualified ProofChecker;
import qualified Polynomial;
import qualified Option;
import qualified Trs;
import qualified Integer;
import qualified ErrorMonad;
import qualified Product_Type;
import qualified HOL;

data Xml = Xml [Char] [([Char], [Char])] [Xml.Xml] (Maybe [Char]);

wspace :: [Char];
wspace = List.append Show.newline (List.append Show.tab [' ']);

trim :: [Char] -> [Char];
trim (c : s) = (if List.member c Xml.wspace then Xml.trim s else c : s);
trim [] = [];

eq_xml :: Xml.Xml -> Xml.Xml -> Bool;
eq_xml (Xml.Xml list1 list2 list3 option) (Xml.Xml list1' list2' list3' option')
  = list1 == list1' &&
      (list2 == list2' && (list3 == list3' && option == option'));

instance Eq Xml.Xml where {
  a == b = Xml.eq_xml a b;
};

data Xmldoc = Xmldoc [Char] Xml.Xml;

data XtcTrs =
  Xtctrs [(Term.Term [Char] [Char], Term.Term [Char] [Char])]
    [([Char], Nat.Nat)] (Maybe Xml.Xml);

dquote :: [Char];
dquote = ['\"'];

parseError :: forall a. [Char] -> [Char] -> Util.Eithera [Char] a;
parseError s allowed =
  ErrorMonad.errora
    (List.append
      ['e', 'x', 'p', 'e', 'c', 't', 'e', 'd', ' ', 's', 'y', 'm', 'b', 'o',
        'l', 's', ' ']
      (List.append allowed
        (List.append [' ', 'b', 'u', 't', ' ', 'f', 'o', 'u', 'n', 'd', ' ']
          (List.takea (Integer.nat 30) s))));

consume2 ::
  [[Char]] -> [[Char]] -> [Char] -> Util.Eithera [Char] ([Char], [Char]);
consume2 [] allowed s =
  Xml.parseError s
    (List.concata (List.mapa (\ sa -> List.append sa [',']) allowed));
consume2 (t : asa) allowed s =
  (if List.mapa snd (List.zipa t s) == t
    then ErrorMonad.returna (t, Xml.trim (List.dropa (List.size_list t) s))
    else Xml.consume2 asa allowed s);

consume :: [[Char]] -> [Char] -> Util.Eithera [Char] ([Char], [Char]);
consume allowed s = Xml.consume2 allowed allowed s;

letters :: [Char];
letters =
  ['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o',
    'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', 'A', 'B', 'C', 'D',
    'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S',
    'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '_', '0', '1', '2', '3', '4', '5', '6',
    '7', '8', '9', '&', ';', ':'];

showAtt :: ([Char], [Char]) -> [Char];
showAtt (a, v) =
  List.append a
    (List.append ['='] (List.append Xml.dquote (List.append v Xml.dquote)));

digit2nat :: Char -> Util.Eithera [Char] Nat.Nat;
digit2nat x =
  (if x == '0' then ErrorMonad.returna Nat.Zero_nat
    else (if x == '1' then ErrorMonad.returna Nat.one_nat
           else (if x == '2' then ErrorMonad.returna (Integer.nat 2)
                  else (if x == '3' then ErrorMonad.returna (Integer.nat 3)
                         else (if x == '4'
                                then ErrorMonad.returna (Integer.nat 4)
                                else (if x == '5'
                                       then ErrorMonad.returna (Integer.nat 5)
                                       else (if x == '6'
      then ErrorMonad.returna (Integer.nat 6)
      else (if x == '7' then ErrorMonad.returna (Integer.nat 7)
             else (if x == '8' then ErrorMonad.returna (Integer.nat 8)
                    else (if x == '9' then ErrorMonad.returna (Integer.nat 9)
                           else ErrorMonad.errora
                                  (List.append [x]
                                    [' ', 'i', 's', ' ', 'n', 'o', 't', ' ',
                                      'a', ' ', 'd', 'i', 'g', 'i',
                                      't'])))))))))));

string2natHelp :: Nat.Nat -> [Char] -> Util.Eithera [Char] Nat.Nat;
string2natHelp n [] = ErrorMonad.returna n;
string2natHelp n (d : s) =
  ErrorMonad.bind (Xml.digit2nat d)
    (\ m ->
      Xml.string2natHelp (Nat.plus_nat (Nat.times_nat (Integer.nat 10) n) m) s);

string2nat :: [Char] -> Util.Eithera [Char] Nat.Nat;
string2nat s =
  (if List.nulla s
    then ErrorMonad.errora
           ['c', 'a', 'n', 'n', 'o', 't', ' ', 'c', 'o', 'n', 'v', 'e', 'r',
             't', ' ', 'e', 'm', 'p', 't', 'y', ' ', 's', 't', 'r', 'i', 'n',
             'g', ' ', 'i', 'n', 't', 'o', ' ', 'n', 'u', 'm', 'b', 'e', 'r']
    else Xml.string2natHelp Nat.Zero_nat s);

showAtts :: [([Char], [Char])] -> [Char];
showAtts asa =
  List.foldra (\ a s -> List.append [' '] (List.append (Xml.showAtt a) s)) asa
    [];

showXMLindent :: [Char] -> Nat.Nat -> Xml.Xml -> [Char];
showXMLindent ind i (Xml.Xml n a c t) =
  List.concata
    [ind, ['<'], n, Xml.showAtts a,
      (if List.nulla c && Option.is_none t then ['/', '>']
        else List.concata
               [['>'],
                 (if not (List.nulla c)
                   then List.append
                          (List.concata
                            (List.mapa
                              (Xml.showXMLindent
                                (List.append ind (List.replicatea i ' ')) i)
                              c))
                          ind
                   else []),
                 (if Option.is_none t then [] else Option.the t), ['<', '/'], n,
                 ['>']])];

sshow_xml :: Xml.Xml -> [Char];
sshow_xml xml = Xml.trim (Xml.showXMLindent Show.newline (Integer.nat 2) xml);

xml2text :: [Char] -> Xml.Xml -> Util.Eithera [Char] [Char];
xml2text reqName (Xml.Xml name atts cs text) =
  (if name == reqName &&
        (List.nulla atts &&
          (List.nulla cs &&
            (not (Option.is_none text) && not (List.nulla (Option.the text)))))
    then ErrorMonad.returna (Option.the text)
    else ErrorMonad.errora
           (List.concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 'e', 'x', 't',
                'r', 'a', 'c', 't', ' ', 't', 'e', 'x', 't', ' ', 'f', 'o', 'r',
                ' '],
               reqName, [' ', 'f', 'r', 'o', 'm', ' ', ' '], Show.newline,
               Xml.sshow_xml (Xml.Xml name atts cs text)]));

xml2nat :: [Char] -> Xml.Xml -> Util.Eithera [Char] Nat.Nat;
xml2nat reqName x = ErrorMonad.bind (Xml.xml2text reqName x) Xml.string2nat;

xml2var :: forall a. Xml.Xml -> Util.Eithera [Char] (Term.Term a [Char]);
xml2var x =
  ErrorMonad.bind (Xml.xml2text ['v', 'a', 'r'] x)
    (ErrorMonad.returna . Term.Var);

xml_sig :: [([Char], Nat.Nat)] -> Xml.Xml;
xml_sig fas =
  Xml.Xml ['s', 'i', 'g', 'n', 'a', 't', 'u', 'r', 'e'] []
    (List.mapa
      (\ (a @ (f, aa)) ->
        Xml.Xml ['f', 'u', 'n', 'c', 's', 'y', 'm'] []
          [Xml.Xml ['n', 'a', 'm', 'e'] [] [] (Just f),
            Xml.Xml ['a', 'r', 'i', 't', 'y'] [] [] (Just (Show.sshow_nat aa))]
          Nothing)
      fas)
    Nothing;

mapMonad ::
  forall a b. (Xml.Xml -> Util.Eithera a b) -> [Xml.Xml] -> Util.Eithera a [b];
mapMonad uu [] = ErrorMonad.returna [];
mapMonad f (xml : rest) =
  (case f xml of {
    Util.Lefta a -> ErrorMonad.errora a;
    Util.Righta a ->
      (case Xml.mapMonad f rest of {
        Util.Lefta aa -> ErrorMonad.errora aa;
        Util.Righta asa -> ErrorMonad.returna (a : asa);
      });
  });

splitterFun :: (Char -> Bool) -> [Char] -> ([Char], [Char]);
splitterFun allowed (c : s) =
  (if allowed c
    then let {
           a = Xml.splitterFun allowed s;
           (l, aa) = a;
         } in (c : l, aa)
    else ([], c : s));
splitterFun uu [] = ([], []);

parseText :: [Char] -> Util.Eithera [Char] (Maybe [Char], [Char]);
parseText s =
  let {
    a = Xml.splitterFun (\ c -> not ([c] == ['<'])) s;
    (l, r) = a;
  } in (if List.nulla (Xml.trim l) then ErrorMonad.returna (Nothing, r)
         else ErrorMonad.returna
                (Just (List.rev (Xml.trim (List.rev (Xml.trim l)))), r));

parseAttribute :: [Char] -> Util.Eithera [Char] ([Char], [Char]);
parseAttribute s =
  ErrorMonad.bind (Xml.consume [Xml.dquote] s)
    (\ (a @ (uu, s1)) ->
      let {
        aa = Xml.splitterFun (\ c -> not ([c] == Xml.dquote)) s1;
        (n, s2) = aa;
      } in ErrorMonad.bind (Xml.consume [Xml.dquote] s2)
             (\ (ab @ (uua, s3)) -> ErrorMonad.returna (n, s3)));

splitter :: [Char] -> [Char] -> ([Char], [Char]);
splitter allowed s = Xml.splitterFun (\ c -> List.member c allowed) s;

parseName :: [Char] -> Util.Eithera [Char] ([Char], [Char]);
parseName s =
  let {
    a = Xml.splitter Xml.letters s;
    (l, r) = a;
  } in (if List.nulla l then Xml.parseError s Xml.letters
         else ErrorMonad.returna (l, Xml.trim r));

parseAttributes :: [Char] -> Util.Eithera [Char] ([([Char], [Char])], [Char]);
parseAttributes [] = ErrorMonad.returna ([], []);
parseAttributes (c : s) =
  (if List.member c ['/', '>'] then ErrorMonad.returna ([], c : s)
    else ErrorMonad.bind (Xml.parseName (c : s))
           (\ (a @ (n, s1)) ->
             ErrorMonad.bind (Xml.consume [['=']] s1)
               (\ (aa @ (uu, s2)) ->
                 ErrorMonad.bind (Xml.parseAttribute s2)
                   (\ (ab @ (ac, s3)) ->
                     ErrorMonad.bind (Xml.parseAttributes s3)
                       (\ (b @ (nas, s4)) ->
                         ErrorMonad.returna ((n, ac) : nas, s4))))));

parseXMLList :: [Char] -> Util.Eithera [Char] ([Xml.Xml], [Char]);
parseXMLList s =
  (if List.nulla s || List.takea (Integer.nat 2) s == ['<', '/']
    then ErrorMonad.returna ([], s)
    else ErrorMonad.bind (Xml.consume [['<']] s)
           (\ (a @ (uu, s1)) ->
             ErrorMonad.bind (Xml.parseName s1)
               (\ (aa @ (n, s2)) ->
                 ErrorMonad.bind (Xml.parseAttributes s2)
                   (\ (ab @ (nas, s3)) ->
                     ErrorMonad.bind (Xml.consume [['/', '>'], ['>']] s3)
                       (\ (ac @ (clo, s4)) ->
                         (if clo == ['/', '>']
                           then ErrorMonad.bind (Xml.parseXMLList s4)
                                  (\ (ad @ (li, s5)) ->
                                    ErrorMonad.returna
                                      (Xml.Xml n nas [] Nothing : li, s5))
                           else ErrorMonad.bind (Xml.parseText s4)
                                  (\ (ad @ (t, s5)) ->
                                    ErrorMonad.bind (Xml.parseXMLList s5)
                                      (\ (ae @ (chi, s6)) ->
ErrorMonad.bind (Xml.consume [List.append ['<', '/'] (List.append n ['>'])] s6)
  (\ (af @ (uua, s7)) ->
    ErrorMonad.bind (Xml.parseXMLList s7)
      (\ (ag @ (li, s8)) ->
        ErrorMonad.returna (Xml.Xml n nas chi t : li, s8)))))))))));

parseXML :: [Char] -> Util.Eithera [Char] (Xml.Xml, [Char]);
parseXML s =
  ErrorMonad.bind (Xml.consume [['<']] s)
    (\ (a @ (uu, s1)) ->
      ErrorMonad.bind (Xml.parseName s1)
        (\ (aa @ (n, s2)) ->
          ErrorMonad.bind (Xml.parseAttributes s2)
            (\ (ab @ (nas, s3)) ->
              ErrorMonad.bind (Xml.consume [['/', '>'], ['>']] s3)
                (\ (ac @ (clo, s4)) ->
                  (if clo == ['/', '>']
                    then ErrorMonad.returna
                           (Xml.Xml n nas [] Nothing, Xml.trim s4)
                    else ErrorMonad.bind (Xml.parseText s4)
                           (\ (ad @ (t, s5)) ->
                             ErrorMonad.bind (Xml.parseXMLList s5)
                               (\ (ae @ (chi, s6)) ->
                                 ErrorMonad.bind
                                   (Xml.consume
                                     [List.append ['<', '/']
(List.append n ['>'])]
                                     s6)
                                   (\ (af @ (uua, s7)) ->
                                     ErrorMonad.returna
                                       (Xml.Xml n nas chi t, s7)))))))));

scanUpto :: [Char] -> [Char] -> Util.Eithera [Char] ([Char], [Char]);
scanUpto end [] =
  ErrorMonad.errora
    (List.append
      ['d', 'i', 'd', ' ', 'n', 'o', 't', ' ', 'f', 'i', 'n', 'd', ' ', 'r',
        'e', 'q', 'u', 'i', 'r', 'e', 'd', ' ', 't', 'o', 'k', 'e', 'n', ' ']
      end);
scanUpto end (c : s) =
  (if List.mapa snd (List.zipa end (c : s)) == end
    then ErrorMonad.returna
           (List.takea (List.size_list end) (c : s),
             Xml.trim (List.dropa (List.size_list end) (c : s)))
    else ErrorMonad.bind (Xml.scanUpto end s)
           (\ (a @ (res, rest)) -> ErrorMonad.returna (c : res, rest)));

string2cint :: [Char] -> Util.Eithera [Char] Util.Cint;
string2cint s =
  (case s of {
    [] -> ErrorMonad.errora
            ['c', 'a', 'n', 'n', 'o', 't', ' ', 'c', 'o', 'n', 'v', 'e', 'r',
              't', ' ', 'e', 'm', 'p', 't', 'y', ' ', 's', 't', 'r', 'i', 'n',
              'g', ' ', 'i', 'n', 't', 'o', ' ', 'n', 'u', 'm', 'b', 'e', 'r'];
    h : t ->
      let {
        a = (if h == '-' then (False, t) else (True, s));
        (pos, ss) = a;
      } in ErrorMonad.bind (Xml.string2nat ss)
             (\ n ->
               (if pos || n == Nat.Zero_nat then ErrorMonad.returna (Util.Pos n)
                 else ErrorMonad.returna (Util.Neg n)));
  });

xml2cint :: [Char] -> Xml.Xml -> Util.Eithera [Char] Util.Cint;
xml2cint reqName x = ErrorMonad.bind (Xml.xml2text reqName x) Xml.string2cint;

xml2name :: Xml.Xml -> Util.Eithera [Char] [Char];
xml2name (Xml.Xml name atts cs text) =
  (if name == ['n', 'a', 'm', 'e'] &&
        (List.nulla cs &&
          (not (Option.is_none text) && not (List.nulla (Option.the text))))
    then (if List.nulla atts ||
               atts == [(['s', 'h', 'a', 'r', 'p'], ['f', 'a', 'l', 's', 'e'])]
           then ErrorMonad.returna (Option.the text)
           else ErrorMonad.errora
                  (List.append
                    (List.append
                      ['i', 'n', 'v', 'a', 'l', 'i', 'd', ' ', 'a', 't', 't',
                        'r', 'i', 'b', 'u', 't', 'e', 's', ' ', 'i', 'n', ' ']
                      Show.newline)
                    (Xml.sshow_xml (Xml.Xml name atts cs text))))
    else ErrorMonad.errora
           (List.append
             (List.append
               ['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 'p', 'a', 'r',
                 's', 'e', ' ', 'n', 'a', 'm', 'e', ' ', 'f', 'r', 'o', 'm',
                 ' ']
               Show.newline)
             (Xml.sshow_xml (Xml.Xml name atts cs text))));

xml1element ::
  forall a b.
    [Char] ->
      (Xml.Xml -> Util.Eithera [Char] a) ->
        (a -> b) -> Xml.Xml -> Util.Eithera [Char] b;
xml1element reqName p1 combi (Xml.Xml name atts cs text) =
  (if name == reqName &&
        (List.nulla atts &&
          (List.size_list cs == Nat.one_nat && Option.is_none text))
    then ErrorMonad.bind (p1 (List.nth cs Nat.Zero_nat))
           (ErrorMonad.returna . combi)
    else ErrorMonad.errora
           (List.concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], Show.newline,
               Xml.sshow_xml (Xml.Xml name atts cs text)]));

xml1manyElements ::
  forall a b c.
    [Char] ->
      (Xml.Xml -> Util.Eithera [Char] a) ->
        (Xml.Xml -> Util.Eithera [Char] b) ->
          (a -> [b] -> c) -> Xml.Xml -> Util.Eithera [Char] c;
xml1manyElements reqName p1 p2 combi (Xml.Xml name atts cs text) =
  (if name == reqName &&
        (List.nulla atts && (Option.is_none text && not (List.nulla cs)))
    then ErrorMonad.bind (p1 (List.hd cs))
           (\ a ->
             ErrorMonad.bind (Xml.mapMonad p2 (List.tl cs))
               (ErrorMonad.returna . combi a))
    else ErrorMonad.errora
           (List.concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], Show.newline,
               Xml.sshow_xml (Xml.Xml name atts cs text)]));

xmlOptionsHelp ::
  forall a.
    [(Xml.Xml -> Util.Eithera [Char] a)] ->
      [[Char]] -> Xml.Xml -> Util.Eithera [Char] a;
xmlOptionsHelp [] errs x =
  ErrorMonad.errora
    (List.append
      (List.concata
        [['n', 'o', ' ', 'o', 'p', 't', 'i', 'o', 'n', ' ', 'f', 'o', 'u', 'n',
           'd', ' ', 'f', 'o', 'r', ' '],
          Show.newline, Xml.sshow_xml x])
      (List.concata
        (List.mapa
          (\ err ->
            List.concata
              [Show.newline, ['-', '-', '-', '-', '-', '-', '-'], Show.newline,
                err])
          errs)));
xmlOptionsHelp (p : ps) errs x =
  ErrorMonad.catcha (p x)
    (\ err -> Xml.xmlOptionsHelp ps (List.append errs [err]) x);

xmlOptions ::
  forall a.
    [(Xml.Xml -> Util.Eithera [Char] a)] -> Xml.Xml -> Util.Eithera [Char] a;
xmlOptions ps x = Xml.xmlOptionsHelp ps [] x;

xml2term :: Xml.Xml -> Util.Eithera [Char] (Term.Term [Char] [Char]);
xml2term x =
  Xml.xmlOptions
    [Xml.xml2var,
      Xml.xml1manyElements ['f', 'u', 'n', 'a', 'p', 'p'] Xml.xml2name
        (Xml.xml1element ['a', 'r', 'g'] Xml.xml2term id) Term.Fun]
    x;

xml2elements ::
  forall a b c.
    [Char] ->
      (Xml.Xml -> Util.Eithera [Char] a) ->
        (Xml.Xml -> Util.Eithera [Char] b) ->
          (a -> b -> c) -> Xml.Xml -> Util.Eithera [Char] c;
xml2elements reqName p1 p2 combi (Xml.Xml name atts cs text) =
  (if name == reqName &&
        (List.nulla atts &&
          (List.size_list cs == Integer.nat 2 && Option.is_none text))
    then ErrorMonad.bind (p1 (List.nth cs Nat.Zero_nat))
           (\ a ->
             ErrorMonad.bind (p2 (List.nth cs Nat.one_nat))
               (ErrorMonad.returna . combi a))
    else ErrorMonad.errora
           (List.concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], Show.newline,
               Xml.sshow_xml (Xml.Xml name atts cs text)]));

xml2rule ::
  Xml.Xml ->
    Util.Eithera [Char] (Term.Term [Char] [Char], Term.Term [Char] [Char]);
xml2rule =
  Xml.xml2elements ['r', 'u', 'l', 'e']
    (Xml.xml1element ['l', 'h', 's'] Xml.xml2term id)
    (Xml.xml1element ['r', 'h', 's'] Xml.xml2term id) (\ a b -> (a, b));

xml_case ::
  forall t.
    ([Char] -> [([Char], [Char])] -> [Xml.Xml] -> Maybe [Char] -> t) ->
      Xml.Xml -> t;
xml_case f1 (Xml.Xml list1 list2 list3 option) = f1 list1 list2 list3 option;

xml_term ::
  forall c d. (Show.Sshow c, Show.Sshow d) => Term.Term c d -> Xml.Xml;
xml_term (Term.Var x) = Xml.Xml ['v', 'a', 'r'] [] [] (Just (Show.sshow x));
xml_term (Term.Fun f ts) =
  Xml.Xml ['f', 'u', 'n', 'a', 'p', 'p'] []
    (Xml.Xml ['n', 'a', 'm', 'e'] [] [] (Just (Show.sshow f)) :
      List.mapa (\ t -> Xml.Xml ['a', 'r', 'g'] [] [Xml.xml_term t] Nothing) ts)
    Nothing;

fitLength :: forall a. a -> Nat.Nat -> [a] -> [a];
fitLength uu Nat.Zero_nat uv = [];
fitLength a (Nat.Suc v) [] = List.replicatea (Nat.Suc v) a;
fitLength a (Nat.Suc n) (b : asa) = b : Xml.fitLength a n asa;

xmlManyElements ::
  forall a b.
    [Char] ->
      (Xml.Xml -> Util.Eithera [Char] a) ->
        ([a] -> b) -> Xml.Xml -> Util.Eithera [Char] b;
xmlManyElements reqName p combi (Xml.Xml name atts cs text) =
  (if name == reqName && (List.nulla atts && Option.is_none text)
    then ErrorMonad.bind (Xml.mapMonad p cs) (ErrorMonad.returna . combi)
    else ErrorMonad.errora
           (List.concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], Show.newline,
               Xml.sshow_xml (Xml.Xml name atts cs text)]));

xml2rules ::
  Xml.Xml ->
    Util.Eithera [Char] [(Term.Term [Char] [Char], Term.Term [Char] [Char])];
xml2rules = Xml.xmlManyElements ['r', 'u', 'l', 'e', 's'] Xml.xml2rule id;

xmlChange ::
  forall a b.
    (Xml.Xml -> Util.Eithera [Char] a) ->
      (a -> b) -> Xml.Xml -> Util.Eithera [Char] b;
xmlChange p f x = ErrorMonad.bind (p x) (\ a -> ErrorMonad.returna (f a));

xml_rules :: [(Term.Term [Char] [Char], Term.Term [Char] [Char])] -> Xml.Xml;
xml_rules lrs =
  Xml.Xml ['r', 'u', 'l', 'e', 's'] []
    (List.mapa
      (\ (a @ (l, r)) ->
        Xml.Xml ['r', 'u', 'l', 'e'] []
          [Xml.Xml ['l', 'h', 's'] [] [Xml.xml_term l] Nothing,
            Xml.Xml ['r', 'h', 's'] [] [Xml.xml_term r] Nothing]
          Nothing)
      lrs)
    Nothing;

data XtcProblem =
  XTCProblem Xml.XtcTrs
    (Bool, ([Xml.Xml], ([([Char], [Char])], ([([Char], [Char])], [Char]))));

xml4elements ::
  forall a b c d e.
    [Char] ->
      (Xml.Xml -> Util.Eithera [Char] a) ->
        (Xml.Xml -> Util.Eithera [Char] b) ->
          (Xml.Xml -> Util.Eithera [Char] c) ->
            (Xml.Xml -> Util.Eithera [Char] d) ->
              (a -> b -> c -> d -> e) -> Xml.Xml -> Util.Eithera [Char] e;
xml4elements reqName p1 p2 p3 p4 combi (Xml.Xml name atts cs text) =
  (if name == reqName &&
        (List.nulla atts &&
          (List.size_list cs == Integer.nat 4 && Option.is_none text))
    then ErrorMonad.bind (p1 (List.nth cs Nat.Zero_nat))
           (\ a ->
             ErrorMonad.bind (p2 (List.nth cs Nat.one_nat))
               (\ b ->
                 ErrorMonad.bind (p3 (List.nth cs (Integer.nat 2)))
                   (\ c ->
                     ErrorMonad.bind (p4 (List.nth cs (Integer.nat 3)))
                       (ErrorMonad.returna . combi a b c))))
    else ErrorMonad.errora
           (List.concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], Show.newline,
               Xml.sshow_xml (Xml.Xml name atts cs text)]));

xml0elements :: forall a. [Char] -> a -> Xml.Xml -> Util.Eithera [Char] a;
xml0elements reqName combi (Xml.Xml name atts cs text) =
  (if name == reqName &&
        (List.nulla atts && (List.nulla cs && Option.is_none text))
    then ErrorMonad.returna combi
    else ErrorMonad.errora
           (List.concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], Show.newline,
               Xml.sshow_xml (Xml.Xml name atts cs text)]));

xml2context :: Xml.Xml -> Util.Eithera [Char] (Term.Ctxt [Char] [Char]);
xml2context x =
  Xml.xmlOptions
    [Xml.xml0elements ['b', 'o', 'x'] Term.Hole,
      Xml.xml4elements ['f', 'u', 'n', 'C', 'o', 'n', 't', 'e', 'x', 't']
        Xml.xml2name
        (Xml.xmlManyElements ['b', 'e', 'f', 'o', 'r', 'e'] Xml.xml2term id)
        Xml.xml2context
        (Xml.xmlManyElements ['a', 'f', 't', 'e', 'r'] Xml.xml2term id)
        Term.More]
    x;

xml2substitution ::
  Xml.Xml ->
    Util.Eithera [Char] [(Term.Term [Char] [Char], Term.Term [Char] [Char])];
xml2substitution =
  Xml.xmlManyElements
    ['s', 'u', 'b', 's', 't', 'i', 't', 'u', 't', 'i', 'o', 'n']
    (Xml.xml2elements ['s', 'u', 'b', 's', 't', 'E', 'n', 't', 'r', 'y']
      Xml.xml2var Xml.xml2term (\ a b -> (a, b)))
    id;

xml2manyElements ::
  forall a b c d.
    [Char] ->
      (Xml.Xml -> Util.Eithera [Char] a) ->
        (Xml.Xml -> Util.Eithera [Char] b) ->
          (Xml.Xml -> Util.Eithera [Char] c) ->
            (a -> b -> [c] -> d) -> Xml.Xml -> Util.Eithera [Char] d;
xml2manyElements reqName p1 p2 p3 combi (Xml.Xml name atts cs text) =
  (if name == reqName &&
        (List.nulla atts &&
          (Option.is_none text &&
            Nat.less_eq_nat (Integer.nat 2) (List.size_list cs)))
    then ErrorMonad.bind (p1 (List.nth cs Nat.Zero_nat))
           (\ a ->
             ErrorMonad.bind (p2 (List.nth cs Nat.one_nat))
               (\ b ->
                 ErrorMonad.bind (Xml.mapMonad p3 (List.tl (List.tl cs)))
                   (ErrorMonad.returna . combi a b)))
    else ErrorMonad.errora
           (List.concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], Show.newline,
               Xml.sshow_xml (Xml.Xml name atts cs text)]));

xml2trsDisproof ::
  Xml.Xml -> Util.Eithera [Char] (ProofChecker.TRSDisproof [Char] [Char]);
xml2trsDisproof =
  Xml.xmlOptions
    [Xml.xml0elements
       ['n', 'o', 't', 'W', 'e', 'l', 'l', 'F', 'o', 'r', 'm', 'e', 'd']
       ProofChecker.NotWellFormed,
      Xml.xml2manyElements ['l', 'o', 'o', 'p'] Xml.xml2substitution
        Xml.xml2context Xml.xml2term
        (\ sigma c ts -> ProofChecker.Loop (ts, (c, sigma)))];

xml3elements ::
  forall a b c d.
    [Char] ->
      (Xml.Xml -> Util.Eithera [Char] a) ->
        (Xml.Xml -> Util.Eithera [Char] b) ->
          (Xml.Xml -> Util.Eithera [Char] c) ->
            (a -> b -> c -> d) -> Xml.Xml -> Util.Eithera [Char] d;
xml3elements reqName p1 p2 p3 combi (Xml.Xml name atts cs text) =
  (if name == reqName &&
        (List.nulla atts &&
          (List.size_list cs == Integer.nat 3 && Option.is_none text))
    then ErrorMonad.bind (p1 (List.nth cs Nat.Zero_nat))
           (\ a ->
             ErrorMonad.bind (p2 (List.nth cs Nat.one_nat))
               (\ b ->
                 ErrorMonad.bind (p3 (List.nth cs (Integer.nat 2)))
                   (ErrorMonad.returna . combi a b)))
    else ErrorMonad.errora
           (List.concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], Show.newline,
               Xml.sshow_xml (Xml.Xml name atts cs text)]));

xml2redpair ::
  forall a.
    (Xml.Xml -> Util.Eithera [Char] a) ->
      Xml.Xml -> Util.Eithera [Char] (Polynomial.RedPair a);
xml2redpair xml2sname =
  Xml.xmlOptions
    [Xml.xml1element ['r', 'e', 'd', 'P', 'a', 'i', 'r']
       (Xml.xml2manyElements
         ['i', 'n', 't', 'e', 'r', 'p', 'r', 'e', 't', 'a', 't', 'i', 'o', 'n']
         (Xml.xml1element ['t', 'y', 'p', 'e']
           (Xml.xml0elements
             ['l', 'i', 'n', 'e', 'a', 'r', 'P', 'o', 'l', 'y', 'n', 'o', 'm',
               'i', 'a', 'l']
             ())
           id)
         (Xml.xml1element ['d', 'o', 'm', 'a', 'i', 'n']
           (Xml.xml0elements ['n', 'a', 't', 'u', 'r', 'a', 'l', 's'] ()) id)
         (Xml.xml3elements ['i', 'n', 't', 'e', 'r', 'p', 'r', 'e', 't']
           xml2sname (Xml.xml2nat ['a', 'r', 'i', 't', 'y'])
           (Xml.xml1element ['f', 'u', 'n', 'c', 't', 'i', 'o', 'n']
             (Xml.xml1manyElements
               ['l', 'i', 'n', 'e', 'a', 'r', 'P', 'o', 'l', 'y', 'n', 'o', 'm',
                 'i', 'a', 'l']
               (Xml.xml1element ['c', 'o', 'n', 's', 't', 'a', 'n', 't']
                 (Xml.xml2cint ['n', 'u', 'm', 'b', 'e', 'r']) id)
               (Xml.xml1element
                 ['c', 'o', 'e', 'f', 'f', 'i', 'c', 'i', 'e', 'n', 't']
                 (Xml.xml2nat ['n', 'a', 't', 'u', 'r', 'a', 'l']) id)
               (\ a b -> (a, b)))
             id)
           (\ f a poly ->
             (f, (fst poly, Xml.fitLength Nat.Zero_nat a (snd poly)))))
         (\ uu uua pia -> pia))
       Polynomial.NegPolo];

xml1to2elements ::
  forall a b c.
    [Char] ->
      (Xml.Xml -> Util.Eithera [Char] a) ->
        (Xml.Xml -> Util.Eithera [Char] b) ->
          (a -> Maybe b -> c) -> Xml.Xml -> Util.Eithera [Char] c;
xml1to2elements reqName p1 p2 combi (Xml.Xml name atts cs text) =
  (if name == reqName &&
        (List.nulla atts &&
          (Nat.less_eq_nat Nat.one_nat (List.size_list cs) &&
            (Nat.less_eq_nat (List.size_list cs) (Integer.nat 2) &&
              Option.is_none text)))
    then ErrorMonad.bind (p1 (List.nth cs Nat.Zero_nat))
           (\ a ->
             (if List.size_list cs == Integer.nat 2
               then ErrorMonad.bind (p2 (List.nth cs Nat.one_nat))
                      (\ b -> ErrorMonad.returna (combi a (Just b)))
               else ErrorMonad.returna (combi a Nothing)))
    else ErrorMonad.errora
           (List.concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], Show.newline,
               Xml.sshow_xml (Xml.Xml name atts cs text)]));

xml5elements ::
  forall a b c d e f.
    [Char] ->
      (Xml.Xml -> Util.Eithera [Char] a) ->
        (Xml.Xml -> Util.Eithera [Char] b) ->
          (Xml.Xml -> Util.Eithera [Char] c) ->
            (Xml.Xml -> Util.Eithera [Char] d) ->
              (Xml.Xml -> Util.Eithera [Char] e) ->
                (a -> b -> c -> d -> e -> f) ->
                  Xml.Xml -> Util.Eithera [Char] f;
xml5elements reqName p1 p2 p3 p4 p5 combi (Xml.Xml name atts cs text) =
  (if name == reqName &&
        (List.nulla atts &&
          (List.size_list cs == Integer.nat 5 && Option.is_none text))
    then ErrorMonad.bind (p1 (List.nth cs Nat.Zero_nat))
           (\ a ->
             ErrorMonad.bind (p2 (List.nth cs Nat.one_nat))
               (\ b ->
                 ErrorMonad.bind (p3 (List.nth cs (Integer.nat 2)))
                   (\ c ->
                     ErrorMonad.bind (p4 (List.nth cs (Integer.nat 3)))
                       (\ d ->
                         ErrorMonad.bind (p5 (List.nth cs (Integer.nat 4)))
                           (ErrorMonad.returna . combi a b c d)))))
    else ErrorMonad.errora
           (List.concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], Show.newline,
               Xml.sshow_xml (Xml.Xml name atts cs text)]));

xml2sharpName :: Xml.Xml -> Util.Eithera [Char] (Trs.Shp [Char]);
xml2sharpName (Xml.Xml name atts cs text) =
  (if name == ['n', 'a', 'm', 'e'] &&
        (List.nulla cs &&
          (not (Option.is_none text) && not (List.nulla (Option.the text))))
    then (if List.nulla atts ||
               atts == [(['s', 'h', 'a', 'r', 'p'], ['f', 'a', 'l', 's', 'e'])]
           then ErrorMonad.returna (Trs.Plain (Option.the text))
           else (if atts == [(['s', 'h', 'a', 'r', 'p'], ['t', 'r', 'u', 'e'])]
                  then ErrorMonad.returna (Trs.Sharp (Option.the text))
                  else ErrorMonad.errora
                         (List.append
                           (List.append
                             ['i', 'n', 'v', 'a', 'l', 'i', 'd', ' ', 'a', 't',
                               't', 'r', 'i', 'b', 'u', 't', 'e', 's', ' ', 'i',
                               'n', ' ']
                             Show.newline)
                           (Xml.sshow_xml (Xml.Xml name atts cs text)))))
    else ErrorMonad.errora
           (List.append
             (List.append
               ['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 'p', 'a', 'r',
                 's', 'e', ' ', 's', 'h', 'a', 'r', 'p', ' ', 'n', 'a', 'm',
                 'e', ' ', 'f', 'r', 'o', 'm', ' ']
               Show.newline)
             (Xml.sshow_xml (Xml.Xml name atts cs text))));

xml2sharpTerm ::
  Xml.Xml -> Util.Eithera [Char] (Term.Term (Trs.Shp [Char]) [Char]);
xml2sharpTerm x =
  Xml.xmlOptions
    [Xml.xml2var,
      Xml.xml1manyElements ['f', 'u', 'n', 'a', 'p', 'p'] Xml.xml2sharpName
        (Xml.xml1element ['a', 'r', 'g'] Xml.xml2sharpTerm id) Term.Fun]
    x;

xml2sharpRule ::
  Xml.Xml ->
    Util.Eithera [Char]
      (Term.Term (Trs.Shp [Char]) [Char], Term.Term (Trs.Shp [Char]) [Char]);
xml2sharpRule =
  Xml.xml2elements ['r', 'u', 'l', 'e']
    (Xml.xml1element ['l', 'h', 's'] Xml.xml2sharpTerm id)
    (Xml.xml1element ['r', 'h', 's'] Xml.xml2sharpTerm id) (\ a b -> (a, b));

xml2sharpRules ::
  Xml.Xml ->
    Util.Eithera [Char]
      [(Term.Term (Trs.Shp [Char]) [Char], Term.Term (Trs.Shp [Char]) [Char])];
xml2sharpRules =
  Xml.xmlManyElements ['r', 'u', 'l', 'e', 's'] Xml.xml2sharpRule id;

xml2dpProof ::
  Xml.Xml -> Util.Eithera [Char] (ProofChecker.DPProof (Trs.Shp [Char]) [Char]);
xml2dpProof x =
  Xml.xmlOptions
    [Xml.xml0elements ['p', 'I', 's', 'E', 'm', 'p', 't', 'y']
       ProofChecker.PisEmpty,
      Xml.xml3elements ['r', 'e', 'd', 'P', 'a', 'i', 'r', 'P', 'r', 'o', 'c']
        (Xml.xml2redpair Xml.xml2sharpName)
        (Xml.xml1element ['d', 'p', 's'] Xml.xml2sharpRules id) Xml.xml2dpProof
        ProofChecker.RedPairProc,
      Xml.xml4elements
        ['r', 'e', 'd', 'P', 'a', 'i', 'r', 'U', 'r', 'P', 'r', 'o', 'c']
        (Xml.xml2redpair Xml.xml2sharpName)
        (Xml.xml1element ['d', 'p', 's'] Xml.xml2sharpRules id)
        (Xml.xml1element ['u', 's', 'a', 'b', 'l', 'e', 'R', 'u', 'l', 'e', 's']
          Xml.xml2sharpRules id)
        Xml.xml2dpProof ProofChecker.RedPairUrProc,
      Xml.xml4elements
        ['m', 'o', 'n', 'o', 'R', 'e', 'd', 'P', 'a', 'i', 'r', 'P', 'r', 'o',
          'c']
        (Xml.xml2redpair Xml.xml2sharpName)
        (Xml.xml1element ['d', 'p', 's'] Xml.xml2sharpRules id)
        (Xml.xml1element ['t', 'r', 's'] Xml.xml2sharpRules id) Xml.xml2dpProof
        ProofChecker.MonoRedPairProc,
      Xml.xml5elements
        ['m', 'o', 'n', 'o', 'R', 'e', 'd', 'P', 'a', 'i', 'r', 'U', 'r', 'P',
          'r', 'o', 'c']
        (Xml.xml2redpair Xml.xml2sharpName)
        (Xml.xml1element ['d', 'p', 's'] Xml.xml2sharpRules id)
        (Xml.xml1element ['t', 'r', 's'] Xml.xml2sharpRules id)
        (Xml.xml1element ['u', 's', 'a', 'b', 'l', 'e', 'R', 'u', 'l', 'e', 's']
          Xml.xml2sharpRules id)
        Xml.xml2dpProof ProofChecker.MonoRedPairUrProc,
      Xml.xmlManyElements
        ['d', 'e', 'p', 'G', 'r', 'a', 'p', 'h', 'P', 'r', 'o', 'c']
        (Xml.xml1to2elements ['c', 'o', 'm', 'p', 'o', 'n', 'e', 'n', 't']
          (Xml.xml1element ['d', 'p', 's'] Xml.xml2sharpRules id)
          Xml.xml2dpProof (\ dps prfOpt -> (prfOpt, dps)))
        ProofChecker.DepGraphProc]
    x;

xml2trsProof ::
  Xml.Xml -> Util.Eithera [Char] (ProofChecker.TRSProof [Char] [Char]);
xml2trsProof x =
  Xml.xmlOptions
    [Xml.xml0elements ['r', 'I', 's', 'E', 'm', 'p', 't', 'y']
       ProofChecker.RisEmpty,
      Xml.xml2elements ['d', 'p', 'T', 'r', 'a', 'n', 's']
        (Xml.xml1element ['d', 'p', 's'] Xml.xml2sharpRules id) Xml.xml2dpProof
        ProofChecker.DPTrans,
      Xml.xml3elements ['r', 'u', 'l', 'e', 'R', 'e', 'm', 'o', 'v', 'a', 'l']
        (Xml.xml2redpair Xml.xml2name)
        (Xml.xml1element ['t', 'r', 's'] Xml.xml2rules id) Xml.xml2trsProof
        ProofChecker.RuleRemoval]
    x;

xmldoc2proof ::
  Xml.Xmldoc ->
    Util.Eithera [Char]
      (Util.Eithera (ProofChecker.TRSProof [Char] [Char])
        (ProofChecker.TRSDisproof [Char] [Char]));
xmldoc2proof (Xml.Xmldoc header (Xml.Xml name atts cs text)) =
  (if name == ['p', 'r', 'o', 'o', 'f'] &&
        (List.size_list cs == Nat.one_nat && Option.is_none text)
    then Xml.xmlOptions
           [Xml.xmlChange Xml.xml2trsProof Util.Lefta,
             Xml.xmlChange Xml.xml2trsDisproof Util.Righta]
           (List.nth cs Nat.Zero_nat)
    else ErrorMonad.errora
           ['i', 'n', 'v', 'a', 'l', 'i', 'd', ' ', 'r', 'o', 'o', 't', ' ',
             'e', 'l', 'e', 'm', 'e', 'n', 't', ',', ' ', 'e', 'x', 'p', 'e',
             'c', 't', 'i', 'n', 'g', ' ', 'p', 'r', 'o', 'o', 'f', ' ', 'w',
             'i', 't', 'h', ' ', 'e', 'x', 'a', 'c', 't', 'l', 'y', ' ', 'o',
             'n', 'e', ' ', 'c', 'h', 'i', 'l', 'd', ' ', 'a', 'n', 'd', ' ',
             'n', 'o', ' ', 't', 'e', 'x', 't']);

stripXMLHeader :: [Char] -> Util.Eithera [Char] ([Char], [Char]);
stripXMLHeader s =
  (if List.takea (Integer.nat 2) (Xml.trim s) == ['<', '?']
    then ErrorMonad.bind (Xml.scanUpto ['?', '>'] s)
           (\ (a @ (header1, s1)) ->
             ErrorMonad.bind (Xml.stripXMLHeader s1)
               (\ (aa @ (header2, s2)) ->
                 ErrorMonad.returna (List.append header1 header2, Xml.trim s2)))
    else ErrorMonad.returna ([], Xml.trim s));

parseXMLDoc :: [Char] -> Util.Eithera [Char] Xml.Xmldoc;
parseXMLDoc s =
  ErrorMonad.bind (Xml.stripXMLHeader s)
    (\ (a @ (header, rest)) ->
      ErrorMonad.bind (Xml.parseXML rest)
        (\ (aa @ (xml, end)) ->
          (if List.nulla end then ErrorMonad.returna (Xml.Xmldoc header xml)
            else ErrorMonad.errora
                   (List.append
                     ['t', 'h', 'e', 'r', 'e', ' ', 'i', 's', ' ', 't', 'e',
                       'x', 't', ' ', 'a', 'f', 't', 'e', 'r', ' ', 'r', 'o',
                       'o', 't', ' ', 'e', 'l', 'e', 'm', 'e', 'n', 't', ':',
                       ' ']
                     (List.append Show.newline end)))));

parseProof ::
  [Char] ->
    Util.Eithera [Char]
      (Util.Eithera (ProofChecker.TRSProof [Char] [Char])
        (ProofChecker.TRSDisproof [Char] [Char]));
parseProof s = ErrorMonad.bind (Xml.parseXMLDoc s) Xml.xmldoc2proof;

xmlGetName :: Xml.Xml -> [Char];
xmlGetName (Xml.Xml name uu uv uw) = name;

xml2funcsym :: Xml.Xml -> Util.Eithera [Char] ([Char], Nat.Nat);
xml2funcsym =
  Xml.xml2elements ['f', 'u', 'n', 'c', 's', 'y', 'm']
    (Xml.xml2text ['n', 'a', 'm', 'e']) (Xml.xml2nat ['a', 'r', 'i', 't', 'y'])
    (\ a b -> (a, b));

xml2signature :: Xml.Xml -> Util.Eithera [Char] [([Char], Nat.Nat)];
xml2signature =
  Xml.xmlManyElements ['s', 'i', 'g', 'n', 'a', 't', 'u', 'r', 'e']
    Xml.xml2funcsym id;

xml2to3elements ::
  forall a b c d.
    [Char] ->
      (Xml.Xml -> Util.Eithera [Char] a) ->
        (Xml.Xml -> Util.Eithera [Char] b) ->
          (Xml.Xml -> Util.Eithera [Char] c) ->
            (a -> b -> Maybe c -> d) -> Xml.Xml -> Util.Eithera [Char] d;
xml2to3elements reqName p1 p2 p3 combi (Xml.Xml name atts cs text) =
  (if name == reqName &&
        (List.nulla atts &&
          (Nat.less_eq_nat (Integer.nat 2) (List.size_list cs) &&
            (Nat.less_eq_nat (List.size_list cs) (Integer.nat 3) &&
              Option.is_none text)))
    then ErrorMonad.bind (p1 (List.nth cs Nat.Zero_nat))
           (\ a ->
             ErrorMonad.bind (p2 (List.nth cs Nat.one_nat))
               (\ b ->
                 (if List.size_list cs == Integer.nat 3
                   then ErrorMonad.bind (p3 (List.nth cs (Integer.nat 2)))
                          (\ c -> ErrorMonad.returna (combi a b (Just c)))
                   else ErrorMonad.returna (combi a b Nothing))))
    else ErrorMonad.errora
           (List.concata
             [['c', 'o', 'u', 'l', 'd', ' ', 'n', 'o', 't', ' ', 't', 'r', 'a',
                'n', 's', 'f', 'o', 'r', 'm', ' ', 't', 'h', 'e', ' ', 'f', 'o',
                'l', 'l', 'o', 'w', 'i', 'n', 'g', ' ', 'x', 'm', 'l', ' ', 'e',
                'l', 'e', 'm', 'e', 'n', 't', ' ', '(', 'e', 'x', 'p', 'e', 'c',
                't', 'e', 'd', ' '],
               reqName, [')'], Show.newline,
               Xml.sshow_xml (Xml.Xml name atts cs text)]));

xml2xtcTrs :: Xml.Xml -> Util.Eithera [Char] Xml.XtcTrs;
xml2xtcTrs =
  Xml.xml2to3elements ['t', 'r', 's'] Xml.xml2rules Xml.xml2signature
    (\ x ->
      (if Xml.xmlGetName x == ['c', 'o', 'm', 'm', 'e', 'n', 't']
        then ErrorMonad.returna x
        else ErrorMonad.errora
               (List.concata
                 [['u', 'n', 'k', 'n', 'o', 'w', 'n', '/', 'u', 'n', 's', 'u',
                    'p', 'p', 'o', 'r', 't', 'e', 'd', ' ', 't', 'h', 'i', 'r',
                    'd', ' ', 'e', 'l', 'e', 'm', 'e', 'n', 't', ' ', 'i', 'n',
                    ' ', 't', 'r', 's', ' ', 'e', 'l', 'e', 'm', 'e', 'n', 't'],
                   Show.newline, Xml.sshow_xml x])))
    Xml.Xtctrs;

xml_xtcTrs :: Xml.XtcTrs -> Xml.Xml;
xml_xtcTrs (Xml.Xtctrs rls sig comment) =
  Xml.Xml ['t', 'r', 's'] []
    (List.append [Xml.xml_rules rls, Xml.xml_sig sig]
      (case comment of {
        Nothing -> [];
        Just c -> [c];
      }))
    Nothing;

xmlStartTerm :: Xml.Xml;
xmlStartTerm =
  Xml.Xml ['s', 't', 'a', 'r', 't', 't', 'e', 'r', 'm'] []
    [Xml.Xml ['f', 'u', 'l', 'l'] [] [] Nothing] Nothing;

xml_xtcProblem :: Xml.XtcProblem -> Xml.Xml;
xml_xtcProblem (Xml.XTCProblem xtcTrs (start, (ssm, (bef, (aft, header))))) =
  Xml.Xml ['p', 'r', 'o', 'b', 'l', 'e', 'm']
    (List.append bef
      (List.append
        [(['t', 'y', 'p', 'e'],
           ['t', 'e', 'r', 'm', 'i', 'n', 'a', 't', 'i', 'o', 'n'])]
        aft))
    (List.append
      [Xml.xml_xtcTrs xtcTrs,
        Xml.Xml ['s', 't', 'r', 'a', 't', 'e', 'g', 'y'] [] []
          (Just ['F', 'U', 'L', 'L'])]
      (List.append (if start then [Xml.xmlStartTerm] else []) ssm))
    Nothing;

xtc2xmldoc :: Xml.XtcProblem -> Xml.Xmldoc;
xtc2xmldoc (Xml.XTCProblem xtcTrs (start, (ssm, (bef, (aft, header))))) =
  Xml.Xmldoc header
    (Xml.xml_xtcProblem
      (Xml.XTCProblem xtcTrs (start, (ssm, (bef, (aft, header))))));

xtcTrs_case ::
  forall t.
    ([(Term.Term [Char] [Char], Term.Term [Char] [Char])] ->
      [([Char], Nat.Nat)] -> Maybe Xml.Xml -> t)
      -> Xml.XtcTrs -> t;
xtcTrs_case f1 (Xml.Xtctrs list1 list2 option) = f1 list1 list2 option;

sshow_xmldoc :: Xml.Xmldoc -> [Char];
sshow_xmldoc (Xml.Xmldoc h x) = List.append h (Xml.sshow_xml x);

extractElement ::
  forall a. (Eq a) => a -> [a] -> Util.Eithera [Char] ([a], [a]);
extractElement uu [] =
  ErrorMonad.errora ['n', 'o', 't', ' ', 'f', 'o', 'u', 'n', 'd'];
extractElement a (b : bs) =
  (if a == b then ErrorMonad.returna ([], bs)
    else ErrorMonad.bind (Xml.extractElement a bs)
           (\ (aa @ (bef, aft)) -> ErrorMonad.returna (b : bef, aft)));

xmldoc2xtcProblem :: Xml.Xmldoc -> Util.Eithera [Char] Xml.XtcProblem;
xmldoc2xtcProblem (Xml.Xmldoc header xml) =
  let {
    (Xml.Xml name atts cs text) = xml;
  } in (if name == ['p', 'r', 'o', 'b', 'l', 'e', 'm'] &&
             (Nat.less_eq_nat (Integer.nat 2) (List.size_list cs) &&
               (Nat.less_eq_nat Nat.one_nat (List.size_list atts) &&
                 Option.is_none text))
         then (if not (List.nth cs Nat.one_nat ==
                        Xml.Xml ['s', 't', 'r', 'a', 't', 'e', 'g', 'y'] [] []
                          (Just ['F', 'U', 'L', 'L']))
                then ErrorMonad.errora
                       ['u', 'n', 's', 'u', 'p', 'p', 'o', 'r', 't', 'e', 'd',
                         ' ', 'o', 'r', ' ', 'u', 'n', 'k', 'n', 'o', 'w', 'n',
                         ' ', 's', 't', 'r', 'a', 't', 'e', 'g', 'y']
                else ErrorMonad.bind
                       (ErrorMonad.catcha
                         (Xml.extractElement
                           (['t', 'y', 'p', 'e'],
                             ['t', 'e', 'r', 'm', 'i', 'n', 'a', 't', 'i', 'o',
                               'n'])
                           atts)
                         (\ x ->
                           ErrorMonad.errora
                             ['t', 'y', 'p', 'e', ' ', 't', 'e', 'r', 'm', 'i',
                               'n', 'a', 't', 'i', 'o', 'n', ' ', 'a', 't', 't',
                               'r', 'i', 'b', 'u', 't', 'e', ' ', 'n', 'o', 't',
                               ' ', 'f', 'o', 'u', 'n', 'd']))
                       (\ (a @ (bef, aft)) ->
                         ErrorMonad.bind
                           (Xml.xml2xtcTrs (List.nth cs Nat.Zero_nat))
                           (\ xtcTrs ->
                             let {
                               aa = (if Nat.less_nat (Integer.nat 2)
  (List.size_list cs)
  && List.nth cs (Integer.nat 2) == Xml.xmlStartTerm
                                      then (True, List.dropa (Integer.nat 3) cs)
                                      else (False,
     List.dropa (Integer.nat 2) cs));
                               (start, rest) = aa;
                             } in (if List.member
['s', 't', 'a', 'r', 't', 't', 'e', 'r', 'm'] (List.mapa Xml.xmlGetName rest)
                                    then ErrorMonad.errora
   ['s', 't', 'a', 'r', 't', 't', 'e', 'r', 'm', 's', ' ', 'u', 'n', 's', 'u',
     'p', 'p', 'o', 'r', 't', 'e', 'd']
                                    else ErrorMonad.returna
   (Xml.XTCProblem xtcTrs (start, (rest, (bef, (aft, header)))))))))
         else ErrorMonad.errora
                ['r', 'o', 'o', 't', ' ', 'n', 'o', 'd', 'e', ' ', 'm', 'u',
                  's', 't', ' ', 'b', 'e', ' ', 'p', 'r', 'o', 'b', 'l', 'e',
                  'm', ' ', 'a', 'n', 'd', ' ', 'h', 'a', 'v', 'e', ' ', 'a',
                  't', ' ', 'l', 'e', 'a', 's', 't', ' ', 't', 'h', 'e', ' ',
                  't', 'w', 'o', ' ', 'i', 'n', 'i', 't', 'i', 'a', 'l', ' ',
                  'c', 'h', 'i', 'l', 'd', 'r', 'e', 'n', ' ', 't', 'r', 's',
                  ' ', 'a', 'n', 'd', ' ', 's', 't', 'r', 'a', 't', 'e', 'g',
                  'y']);

parseXTCProblem :: [Char] -> Util.Eithera [Char] Xml.XtcProblem;
parseXTCProblem s = ErrorMonad.bind (Xml.parseXMLDoc s) Xml.xmldoc2xtcProblem;

xtcProblem_case ::
  forall t.
    (Xml.XtcTrs ->
      (Bool, ([Xml.Xml], ([([Char], [Char])], ([([Char], [Char])], [Char])))) ->
        t)
      -> Xml.XtcProblem -> t;
xtcProblem_case f1 (Xml.XTCProblem xtcTrs x) = f1 xtcTrs x;

sshow_xtcProblem :: Xml.XtcProblem -> [Char];
sshow_xtcProblem p = Xml.sshow_xmldoc (Xml.xtc2xmldoc p);

}
