(*
Copyright 2009 Christian Sternagel, René Thiemann, Sarah Winkler, Harald Zankl

This file is part of IsaFoR/CeTA.

IsaFoR/CeTA is free software: you can redistribute it and/or modify it under the
terms of the GNU Lesser General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

IsaFoR/CeTA is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along
with IsaFoR/CeTA. If not, see <http://www.gnu.org/licenses/>.
*)
theory CetaVerifier imports ProofChecker Xml 
begin

abbreviation dropWhiteSpace :: "string \<Rightarrow> string" 
where "dropWhiteSpace \<equiv> filter (\<lambda> c. \<not> c mem wspace)"

definition eqWhiteSpace :: "string \<Rightarrow> string \<Rightarrow> bool" ("_ =w _" 50)
where "s1 =w s2 \<equiv> dropWhiteSpace s1 = dropWhiteSpace s2"

fun getTrs :: "xtcProblem \<Rightarrow> (string,string)trel"
where "getTrs (XTCProblem (XTCTRS R sig comment) metaInf) = set R"

datatype certificationResult = CertifiedYes | CertifiedNo | Error string

fun certifyProof :: "string \<Rightarrow> string \<Rightarrow> certificationResult"
where "certifyProof trsString prfString =
  (case (parseXTCProblem trsString 
  \<guillemotright>= (\<lambda> xtcProblem. 
          let parsedTrsString = sshow xtcProblem in
    checkB (trsString =w parsedTrsString) 
           (sConcat [''the parsed TRS does not correspond to the input TRS'', newline, parsedTrsString])
 \<guillemotright> (case xtcProblem of 
         XTCProblem xtcR metaInf \<Rightarrow> 
   (case xtcR of
           XTCTRS R sig comment \<Rightarrow>     
  parseProof prfString
  \<guillemotright>= (\<lambda> prf. case prf of
         Left termPrf \<Rightarrow> 
            (case checkTRSProof ''1'' R termPrf of
               Left err \<Rightarrow> error (showList err)
             | Right _  \<Rightarrow> return True)
       | Right nontermPrf \<Rightarrow> 
            (case checkTRSDisproof ''1'' R nontermPrf of
               Left err \<Rightarrow> error (showList err)
             | Right _  \<Rightarrow> return False)
   )))))
  of Left err \<Rightarrow> Error err | Right b \<Rightarrow> if b then CertifiedYes else CertifiedNo)"

lemma certifyProofSoundI : assumes 
  "certifyProof trsString prfString = CertifiedYes"
shows "\<exists> R. trsString =w sshow R \<and> SN (rstep (getTrs R))"
using assms
proof (cases "parseXTCProblem trsString", simp add: bind_def) 
  case (Right xtcProblem) 
  with assms show ?thesis
  proof (cases "trsString =w sshow xtcProblem")
    case False with Right assms have False 
      by (simp add: Let_def bind_def checkB_def error_def) 
    thus ?thesis ..
  next
    case True 
    show ?thesis 
    proof (cases xtcProblem)
      case (XTCProblem xtcR metaInf)
      show ?thesis
      proof (cases xtcR)
	case (XTCTRS R sig comment)
	from prems have trs: "getTrs xtcProblem = set R" by simp
	from prems show ?thesis
	proof (cases "parseProof prfString", 
	    simp add: Let_def bind_def checkB_def error_def return_def) 
	  case (Right "prf")
	  show ?thesis
	  proof (cases "prf")
	    case (Right nontermPrf)
	    from prems show ?thesis 
	      by (cases "checkTRSDisproof ''1'' R nontermPrf", auto simp add: Let_def bind_def checkB_def error_def return_def)
	  next
	    case (Left termPrf)
	    with prems have "isOK(checkTRSProof ''1'' R termPrf)" 
	      by (cases "checkTRSProof ''1'' R termPrf", auto simp add: Let_def bind_def checkB_def error_def return_def isValue_def)
	    hence "SN(rstep (set R))" by (rule checkTRSProofSound)
	    with trs True show ?thesis by auto 
	  qed
	qed
      qed
    qed
  qed
qed

lemma certifyProofSoundII : assumes 
  "certifyProof trsString prfString = CertifiedNo"
shows "\<exists> R. trsString =w sshow R \<and> \<not> SN (rstep (getTrs R))"
using assms
proof (cases "parseXTCProblem trsString", simp add: bind_def) 
  case (Right xtcProblem) 
  with assms show ?thesis
  proof (cases "trsString =w sshow xtcProblem")
    case False with Right assms have False 
      by (simp add: Let_def bind_def checkB_def error_def) 
    thus ?thesis ..
  next
    case True 
    show ?thesis 
    proof (cases xtcProblem)
      case (XTCProblem xtcR metaInf)
      show ?thesis
      proof (cases xtcR)
	case (XTCTRS R sig comment)
	from prems have trs: "getTrs xtcProblem = set R" by simp
	from prems show ?thesis
	proof (cases "parseProof prfString", 
	    simp add: Let_def bind_def checkB_def error_def return_def) 
	  case (Right "prf")
	  show ?thesis
	  proof (cases "prf")
	    case (Left termPrf)
	    from prems show ?thesis 
	      by (cases "checkTRSProof ''1'' R termPrf", auto simp add: Let_def bind_def checkB_def error_def return_def)
	  next
	    case (Right nontermPrf)
	    with prems have "isOK(checkTRSDisproof ''1'' R nontermPrf)" 
	      by (cases "checkTRSDisproof ''1'' R nontermPrf", auto simp add: Let_def bind_def checkB_def error_def return_def isValue_def)
	    hence "\<not> SN(rstep (set R))" by (rule checkTRSDisproofSound)
	    with trs True show ?thesis by auto 
	  qed
	qed
      qed
    qed
  qed
qed


end
