(*
Copyright 2009 Christian Sternagel, René Thiemann, Sarah Winkler, Harald Zankl

This file is part of IsaFoR/CeTA.

IsaFoR/CeTA is free software: you can redistribute it and/or modify it under the
terms of the GNU Lesser General Public License as published by the Free Software
Foundation, either version 3 of the License, or (at your option) any later
version.

IsaFoR/CeTA is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public License along
with IsaFoR/CeTA. If not, see <http://www.gnu.org/licenses/>.
*)
theory ProofChecker imports Substitution DepGraph Polynomial DpTransImpl Loop begin



text {* 
   A polynomial order is represented by a list of function symbol assignments where 
   every function symbol obtains a pair of a number n and a list of numbers [n1,..,nk].
   The corresponding polynomial interpretation is as follows:
   Pol(f(x1,..,xl)) = n + sum_i ni * xi
        where i ranges from 1 to min(k,l)
*}

datatype ('f,'v)"DPProof" = PisEmpty  
                          | RedPairProc "'f RedPair" "('f,'v)trsL"  "('f,'v) DPProof"
                          | RedPairUrProc "'f RedPair" "('f,'v)trsL" "('f,'v)trsL" "('f,'v) DPProof"
                          | DepGraphProc "(('f,'v)DPProof option \<times> ('f,'v)trsL) list"
                          | MonoRedPairProc "'f RedPair" "('f,'v)trsL" "('f,'v)trsL" "('f,'v) DPProof" 
                          | MonoRedPairUrProc "'f RedPair" "('f,'v)trsL" "('f,'v)trsL" "('f,'v)trsL" "('f,'v)DPProof" 



text {* appropriate wrappers for loop *}
types ('f,'v) subL = "(('f,'v)term \<times>('f,'v)term) list"
types ('f,'v) loopL = "('f,'v)term list \<times> ('f,'v)ctxt \<times> ('f,'v)subL"

fun "checkLoop" :: "('f :: sshow,'v :: sshow)trsL \<Rightarrow> ('f,'v)loopL \<Rightarrow> sstring list monad" where
 "checkLoop R (ts,C,sub) = ((case Loop.subst_safe [] sub \<iota> of None \<Rightarrow> error ''substitution not unique''
                                                           | Some \<sigma> \<Rightarrow> check_loop (ts,C,\<sigma>) R)
      \<leftarrow> (\<lambda>s. [''error in certification of loop'',s]))"


datatype ('f,'v)"TRSProof" = DPTrans "('f,'v)dpL"  "('f shp, 'v) DPProof" 
                           | RuleRemoval "'f RedPair" "('f,'v)trsL" "('f,'v)TRSProof"
                           | RisEmpty

datatype ('f,'v)"TRSDisproof" = Loop "('f,'v)loopL" | NotWellFormed


fun indent :: "sstring list \<Rightarrow> sstring list"
where "indent xs = (newline # (map (\<lambda> s. ''  '' @ s) xs))" 


function checkDPProof :: "sstring \<Rightarrow> ('f,'v)trsL \<Rightarrow> ('f,'v)trsL \<Rightarrow> ('f :: sshow, 'v :: sshow)DPProof \<Rightarrow> sstring list monad" 
where 
   "checkDPProof i P R PisEmpty = 
            (case P of Nil \<Rightarrow> return () | Cons _ _ \<Rightarrow> 
                     error ((i +++ '': P is not empty in the following DP-problem '') # showDPP P R))"  
|  "checkDPProof i P R (RedPairProc redp rP prf)  = (
         createRedPairProc redp P rP R 
           \<leftarrow> (\<lambda> s. ((i +++ '': error when applying the reduction pair processor to switch from '') # (showDPP P R @ ''to the DP problem '' # showDPP rP R @ s))) 
       \<guillemotright> checkDPProof (i +++ ''.1'') rP R prf
           \<leftarrow> (\<lambda> s. (i +++ '': error below the reduction pair processor'') # (indent s))
)" 
|  "checkDPProof i P R (RedPairUrProc redp rP ur prf)  = (
         createRedPairUrAfProc redp P rP R ur
           \<leftarrow> (\<lambda> s. ((i +++ '': error when applying the reduction pair processor (with usable rules and af) to switch from '') # (showDPP P R @ ''to the DP problem '' # showDPP rP R @ s))) 
       \<guillemotright> checkDPProof (i +++ ''.1'') rP R prf
           \<leftarrow> (\<lambda> s. (i +++ '': error below the reduction pair processor'') # (indent s))
)" 
| "checkDPProof i P R (DepGraphProc edpts) = (
       checkDepGraphProc P (map (\<lambda> x. ((case fst x of Some _ \<Rightarrow> True | None \<Rightarrow> False), snd x)) edpts) R
           \<leftarrow> (\<lambda> s. (i +++ '': error while trying to perform SCC-decomposition  on '') # 
                          (showDPP P R @  s))
     \<guillemotright> checkAllMIndex (\<lambda> j rp. case fst rp of Some prf \<Rightarrow> Suc j | None \<Rightarrow> j) (\<lambda> j rp. case fst rp of Some prf \<Rightarrow> checkDPProof (i +++ ''.'' +++ sshow j) (snd rp) R prf | None \<Rightarrow> OK) edpts
           \<leftarrow> (\<lambda> s. (i +++ '': error below the dependency graph processor'') # (indent s))
)"
|  "checkDPProof i P R (MonoRedPairProc redp rP rR prf)  = (
         createMonoRedPairProc redp P rP R rR
           \<leftarrow> (\<lambda> s. ((i +++ '': error when applying the monotonic reduction pair processor to switch from '') # (showDPP P R @ ''to the DP problem '' # showDPP rP rR @ s))) 
       \<guillemotright> checkDPProof (i +++ ''.1'') rP rR prf
           \<leftarrow> (\<lambda> s. (i +++ '': error below the monotonic reduction pair processor'') # (indent s))
)"
|  "checkDPProof i P R (MonoRedPairUrProc redp rP rR ur prf)  = (
         createMonoRedPairUrProc redp P rP R rR ur
           \<leftarrow> (\<lambda> s. ((i +++ '': error when applying the monotonic reduction pair processor (with usable rules) to switch from '') # (showDPP P R @ ''to the DP problem '' # showDPP rP rR @ s))) 
       \<guillemotright> checkDPProof (i +++ ''.1'') rP rR prf
           \<leftarrow> (\<lambda> s. (i +++ '': error below the monotonic reduction pair (with usable rules) processor'') # (indent s))
)"
by (pat_completeness, auto)
termination "checkDPProof"
proof 
  let ?R = "measures [(\<lambda> (i,P,R,prf). size prf)]"
  show "wf ?R" by auto
next
  fix i P R redp rP "prf" 
  show "((i +++ ''.1'', rP, R, prf), i, P, R, RedPairProc redp rP prf) \<in> measures [\<lambda>(i, P, R, prf). size prf]" by auto
next
  fix i P R redp rP ur "prf"
  show "((i +++ ''.1'', rP, R, prf), i, P, R, RedPairUrProc redp rP ur prf) \<in> measures [\<lambda>(i, P, R, prf). size prf]" by auto
next
  fix i P R redp rP rR "prf"
  show "((i +++ ''.1'', rP, rR, prf), i, P, R, MonoRedPairProc redp rP rR prf) \<in> measures [\<lambda>(i, P, R, prf). size prf]" by auto
next
  fix i P R redp rP rR ur "prf"
  show "((i +++ ''.1'', rP, rR, prf), i, P, R, MonoRedPairUrProc redp rP rR ur prf) \<in> measures [\<lambda>(i, P, R, prf). size prf]" by auto
next
  fix i and P R :: "('f::sshow, 'v:: sshow)trsL" and 
    edpts :: "(('f,'v)DPProof option \<times> ('f,'v)trsL) list" and n x and "prf" :: "('f,'v)DPProof"
  assume elem: "x \<in> set edpts" and "fst x = Some prf"
  hence "(option_size size) (fst x) = Suc (size prf)" by auto
  hence "prod_size (option_size size) (list_size (prod_size (term_size (\<lambda>x. 0) (\<lambda>x. 0)) (term_size (\<lambda>x. 0) (\<lambda>x. 0)))) x > (size prf)"
    (is "?m x > size prf")
    by (cases x, auto)   
  with elem have "size prf < ((list_size ?m) edpts)" 
  proof (induct edpts, simp)
    case (Cons y ys)
    thus ?case by (cases "x = y", auto)
  qed
  thus "((i +++ ''.'' +++ sshow n, snd x, R, prf), i, P, R, DepGraphProc edpts) \<in> measures [\<lambda>(i, P, R, prf). size prf]" by auto
qed

  


lemma checkDPProofSound[simp]  : fixes P :: "('f :: sshow,'v :: sshow)trsL"
  assumes "isOK(checkDPProof i P R prf)"
  shows "finite_dpp (set P, set R)"
using assms
proof (induct rule:checkDPProof.induct)
  case (1 i P R) 
  thus "finite_dpp (set P, set R)" unfolding finite_dpp_def by (cases P, auto)
next
  case (2 i P R redp rP prof) 
  thus ?case by auto
next
  case (3 i P R redp rP ur prof) 
  thus ?case by auto
next
  case (4 i P R edpts) 
  let ?mc = "\<lambda> edpts. (map (\<lambda> x. ((case fst x of Some _ \<Rightarrow> True | None \<Rightarrow> False), snd x)) edpts)"
  let ?edps = "?mc edpts"
  show ?case 
  proof (rule checkDepGraphProcSound, auto)
    from 4 show "isOK(checkDepGraphProc P ?edps R)" by (auto)
  next
    fix a rp
    assume elem: "(True, rp) \<in> set ?edps"
    from 4 have "isOK ( checkAllMIndex (\<lambda> j rp. case fst rp of Some prf \<Rightarrow> Suc j | None \<Rightarrow> j) (\<lambda> j rp. case fst rp of Some prf \<Rightarrow> checkDPProof (i +++ ''.'' +++ sshow j) (snd rp) R prf | None \<Rightarrow> OK) edpts)" (is "isOK ( checkAllMIndex ?g ?f edpts)") by auto
    hence "\<forall> prfrP \<in> set edpts. \<exists> n. isOK ( ?f n prfrP)" by (rule checkAllMIndexSound)
    with 4 have fin: "\<forall> prf rP. (Some prf, rP) \<in> set edpts \<longrightarrow> finite_dpp (set rP, set R)" by auto
    from elem obtain sub where "set sub \<subseteq> set edpts \<and> (True, rp) \<in> set (?mc sub)" by blast
    thus "finite_dpp (set rp, set R)"
    proof (induct sub, simp)
      case (Cons edp subb)
      with fin show ?case 
	by (cases "edp", cases "fst edp", simp, cases "snd edp = rp", auto)
    qed
  qed
next
  case (5 i P R redp rP rR prof) 
  thus ?case by auto
next
  case (6 i P R redp rP rR ur prof) 
  thus ?case by auto
qed


declare checkDPProof.simps[simp del]


fun checkTRSProof :: "sstring \<Rightarrow> ('f :: sshow, 'v :: sshow)trsL \<Rightarrow> ('f,'v) TRSProof \<Rightarrow> sstring list monad"
where "checkTRSProof i r (DPTrans P prf) =  (
    checkDPs r P
      \<leftarrow> (\<lambda> s. (i +++ '': error when switching from the TRS'') # 
                 ( showTRS r @  (''to the initial DP problem'' # showDPP P (computeID r) @  s)))
  \<guillemotright> checkDPProof (i +++ ''.1'') P (computeID r) prf
      \<leftarrow> (\<lambda> s. (i +++ '': error below switch to dependency pairs'') # (indent s))
)"
  | "checkTRSProof i r (RuleRemoval rp rR prf) = (
      createRuleRemovalProc rp r rR
           \<leftarrow> (\<lambda> s. ((i +++ '': error in rule removal when switching from the TRS'') # (showTRS r @ (''to the TRS'' # (showTRS rR @ s))))) 
       \<guillemotright> checkTRSProof (i +++ ''.1'') rR prf
           \<leftarrow> (\<lambda> s. (i +++ '': error below the rule removal processor'') # (indent s))
)"
  | "checkTRSProof i r RisEmpty = checkB (r = []) ([i +++ '': R is not empty, but it is the TRS''] @ showTRS r)"

fun checkTRSDisproof :: "string \<Rightarrow> ('f :: sshow, 'v :: sshow)trsL \<Rightarrow> ('f,'v) TRSDisproof \<Rightarrow> sstring list monad"
where "checkTRSDisproof i r (Loop l) = (checkLoop r l \<leftarrow> (\<lambda> s. (i @ '': error in loop proof'') # s))"
    | "checkTRSDisproof i r NotWellFormed = (checkNotWellFormed r \<leftarrow> (\<lambda> s. (i @ '': error in not-well-formed proof'') # s))"

lemma checkTRSProofSound  : 
  assumes check: "isOK(checkTRSProof i r prof)"
  shows "SN (rstep (set r))"
using assms
proof (induct "prof" arbitrary:r i)
  case (DPTrans P dpprf)
  hence dpsSound: "isOK(checkDPs r P)" and rec: "isOK(checkDPProof (i +++ ''.1'') P (computeID r) dpprf)" by auto
  from rec have "finite_dpp (set P, set (computeID r))" by (rule checkDPProofSound)
  with dpsSound show "SN (rstep (set r))" by (simp)
next
  case (RuleRemoval rp rR prof')
  hence ok:"isOK (checkTRSProof i r (RuleRemoval rp rR prof'))" and IH:"\<And>r' i'. isOK (checkTRSProof i' r' prof') \<Longrightarrow> SN (rstep (set r'))" by auto
  from ok have rr_ok:"isOK(createRuleRemovalProc rp r rR)" and rec_ok:"isOK(checkTRSProof (i +++ ''.1'') rR prof')" by auto
  from rec_ok IH have "SN (rstep (set rR))" by auto
  with rr_ok show "SN (rstep (set r))" using createRuleRemovalProcSound by auto
next
  case RisEmpty
  hence "r = []" by auto
  hence "\<forall> l rr p \<sigma>. rstep_r_p_s (set r) (l,rr) p \<sigma> = {}" unfolding rstep_r_p_s_def by auto
  with rstep_iff_rstep_r_p_s have "rstep (set r) = {}" by best 
  thus "SN (rstep (set r))" unfolding SN_ars_def SN_elt_def by blast
qed

lemma checkTRSDisproofSound :
 assumes "isOK(checkTRSDisproof i r prf)"
 shows "\<not> SN (rstep (set r))"
using assms 
proof (cases "prf")
  case NotWellFormed
  with assms show ?thesis by simp
next
  case (Loop x)
  show ?thesis 
  proof (cases x)
    fix ts C \<sigma> 
    assume "x = (ts,C,\<sigma>)"
    with assms Loop have "isOK(checkLoop r (ts, C, \<sigma>))" by simp
    thus ?thesis
      by (cases "subst_safe [] \<sigma> \<iota>") auto 
  qed
qed

end
